unit WDosPeasInOuts;

interface

uses
  SysUtils, Classes, WDosPlcUtils, WDosPlcs, WDosPeas;

type
  TwdxPeasInOut = class (TCustomInOut)
  private
    FPeasCtrl: TwdxPeasCtrl;
  protected
    procedure BeforeDestroy; override;
    procedure DoClose; override;
    procedure DoOpen; override;
    function GetController: TwdxPeasIO;
    function GetDigit: Integer;
    function GetMaxInp: Byte;
    function GetMinOut: TPeasAddr;
    procedure ProcessInput(var IoData: TIoData); override;
    procedure ProcessOutput(var IoData: TIoData); override;
    procedure ResetOutput; override;
    procedure SetController(Value: TwdxPeasIO);
    procedure SetDigit(Value: Integer);
    procedure SetMaxInp(Value: Byte);
    procedure SetMinOut(Value: TPeasAddr);
    procedure SetState(Value: TInOutState); override;
  public
    constructor Create(aOwner: TComponent); override;
  published
    property Controller: TwdxPeasIO read GetController write SetController;
    property Digit: Integer read GetDigit write SetDigit;
    property MaxInp: Byte read GetMaxInp write SetMaxInp;
    property MinOut: TPeasAddr read GetMinOut write SetMinOut;
  end;
  
implementation

{
************************************* TwdxPeasInOut **************************************
}
constructor TwdxPeasInOut.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  { create own objects }
  FPeasCtrl :=TwdxPeasCtrl.Create(Self);
end;

procedure TwdxPeasInOut.BeforeDestroy;
begin
  FPeasCtrl.Free;
end;

procedure TwdxPeasInOut.DoClose;
begin
  FPeasCtrl.Close;
end;

procedure TwdxPeasInOut.DoOpen;
begin
  FPeasCtrl.Open;
end;

procedure TwdxPeasInOut.ProcessInput(var IoData: TIoData);
var
  I: Integer;
begin
  with FPeasCtrl do
  begin
    {Fehlermeldung der PEAS P-Baugruppe prfen}
    if Active and FPeasCtrl.ErrorIUE then
      Error(ecsHardware);
    {Eingnge lesen}
    Addr :=0;
    for I :=0 to MaxInp do
    begin
      Byte(IoData[I, False]) :=Input;
      IncAddr;
    end;
  end; {with}           
end;

procedure TwdxPeasInOut.ProcessOutput(var IoData: TIoData);
var
  I: Integer;
begin
  with FPeasCtrl do
    {Funktionen im Run-Modus}
    if State =iosRun then
    begin
      {Zeitberwachung aktualisieren}
      Refresh;
      {Ausgnge aktualisieren}
      Addr :=61;
      for I :=61 downto MinOut do
      begin
        Output :=Byte(IoData[I, True]);
        DecAddr;
      end;
    end
  else ResetOutput;
end;

procedure TwdxPeasInOut.ResetOutput;
var
  I: Integer;
begin
  with FPeasCtrl do
  begin
    { reset all outputs }
    Addr :=61;
    for I :=61 downto MinOut do
    begin
      Output :=0;
      DecAddr;
    end;
  end;
end;

procedure TwdxPeasInOut.SetState(Value: TInOutState);
begin
  if State <> Value then
  begin
    with FPeasCtrl do
      case Value of
      iosError:
        {im Fehler-Modus keine Ausgaben an die PEAS}
        ;
      iosRun:
        {PEAS in den Run-Zustand bringen}
        Status :=Status +[psL];
      else
        {PEAS in den Halt-Zustand bringen}
        Status :=Status -[psL];
      end;
  end;
  inherited SetState(Value);
end;

function TwdxPeasInOut.GetController: TwdxPeasIO;
begin
  Result := FPeasCtrl.Controller;
end;

procedure TwdxPeasInOut.SetController(Value: TwdxPeasIO);
begin
  FPeasCtrl.Controller := Value;
end;

function TwdxPeasInOut.GetDigit: Integer;
begin
  Result := FPeasCtrl.Digit;
end;

procedure TwdxPeasInOut.SetDigit(Value: Integer);
begin
  FPeasCtrl.Digit := Value;
end;

function TwdxPeasInOut.GetMaxInp: Byte;
begin
  Result := FPeasCtrl.MaxInp;
end;

procedure TwdxPeasInOut.SetMaxInp(Value: Byte);
begin
  FPeasCtrl.MaxInp := Value;
end;

function TwdxPeasInOut.GetMinOut: TPeasAddr;
begin
  Result := FPeasCtrl.MinOut;
end;

procedure TwdxPeasInOut.SetMinOut(Value: TPeasAddr);
begin
  FPeasCtrl.MinOut := Value;
end;

end.
